# -*- coding: utf-8 -*-
"""
Created on Sun Dec 25 16:18:33 2022

@author: farafehizoro
"""
from .wrsi import Wrsi

class wrsi_dekadal(Wrsi):
    """
    Class to calculate dekadal wrsi
    valid method: "Original" or "Modified"
    """
    def __init__(self, ETa, ETc, method = "Original", rain = []):
        
        Wrsi.__init__(self, ETa, ETc, method, rain)
        self.wrsi = []

    def calculate_wrsi_dekadal(self):
        """
        calculate wrsi for dekadal data

        Returns
        -------
        a list containing the dekadal wrsi data .

        """
        self.wrsi = []
        self._update_status()
        if (not self._same_length_ET):
            print ("ETa and ETc do not have the same length. \n Wrsi cannot be calculated")
            return self.wrsi
        #check if there are negative number
        
        if (self._ETa_negative or self._ETc_negative): 
            print("Negative number in ETa or ETc, please check your data. \nWrsi cannot be calculated")
            return self.wrsi
        
        #check rain data lenght and value (>0)
        if (self._with_rain):  #verify the rain data
            if(not self._same_length_rain):
                print ("Rain does not have the same length as ETa and ETc. Rain data is not considered for wrsi calculation.")
                self._rain = []
                self._with_rain = False
            if (self._rain_negative):
                print("Negative value for Rain data, please check your data. Rain data is not considered for wrsi calculation.")
                self._rain = []
                self._with_rain = False
        
        if (self._method == "Original"): #mbol tsy implémenter ko ny fahafatesan'ilay voly : to do
            self.wrsi = self._wrsi_original_dekadal()
        elif(self._method == "Modified"): 
            self.wrsi = self._wrsi_modified_dekadal()
        else: 
            print("Invalid method. \nPlease update the method to use with .update_method()")
            return self.wrsi
        
        self.check_parameter()
        return self.wrsi
    
    def _wrsi_original_dekadal(self):
        """
        calculate wrsi using original method for dekadal data

        Returns
        -------
        a list containing dekadal wrsi data .

        """
        wrsi_original = []
           
        wrsi_temp = 100 
        ETc_tot = sum (self._ETc)
        for i in range(len(self._ETa)): 
            if ( self._ETa[i] < self._ETc[i]): #if there was water deficit
                diff = self._ETc[i] - self._ETa[i]
                wrsi_temp = wrsi_temp - 100 * (diff / ETc_tot) #water déficit
                if(self._with_rain): 
                    if(self._water_excess_dek(self._ETa[i], self._rain[i])):
                        wrsi_temp = wrsi_temp - 3    
                            
            wrsi_original.append(wrsi_temp)
        return wrsi_original
    
    def _wrsi_modified_dekadal(self):
        """
        calculate dekadal wrsi with modified method

        Returns
        -------
        wrsi: list of dekadal wrsi.

        """
        wrsi_modified = []
        ETa_cumul = 0
        ETc_cumul = 0
        excess_number = 0
        for i in range(len(self._ETa)):
            ETa_cumul += self._ETa[i]
            ETc_cumul += self._ETc[i]
            if(self._with_rain):
                if(self._water_excess_dek(self._ETa[i], self._rain[i])):
                    excess_number += 1  #on compte les nombre de dek avec exces
            wrsi_temp = 100 * ETa_cumul / ETc_cumul - (excess_number * 3)
            wrsi_modified.append(wrsi_temp)
            
        return wrsi_modified
    
    def _water_excess_dek(self, ET_a, RR):
        """
        Method to determine if there was a water excess within the dekad 
        Parameters
        ----------
        ETa : list
            evapotranpiration.
        RR : list
            evapptranspiration.

        Returns
        -------
        bool
            true if there is water excess.
        """
        water_excess = RR - ET_a
        if (water_excess > 100): 
            return True
        return False


# -*- coding: utf-8 -*-
"""
Created on Sun Dec 25 12:18:32 2022

@author: Farafehizoro Ramanjatonirina
"""
class Wrsi:
    """
    General class to calculate the water requirement satisfaction index of annual crop
    
    Attributes: 
        * ETa (list): actual evapotranspiration for the crop
        * ETc (list): potentail evapotranspiration for the crop (max ETa if there is no water shortage)
        * method (str): the method to use "Original" or "Modified"
        * rain (list): rainfall data (optional)
    """
    def __init__(self, ETa, ETc, method = "Original", rain = []):
        self._ETa = ETa
        self._ETc = ETc
        self._method = method
        self._rain = rain
        self._with_rain = False
        self._same_length_ET = False
        self._ETa_negative = False
        self._ETc_negative = False
        self._rain_negative = False
        self._same_length_rain = False
        self._update_status()
        
    def update_method(self, method = "Original"):
        """
        Parameters
        ----------
        method : string, optional
            update the method used to calculate wrsi. The default is "Original".
            Valid option: "Original" or "Modified"

        Returns
        -------
        None.

        """
        self._method = method
        
    def _update_status(self):
        """
        Method to check the data and update the attribute about the data status

        Returns
        -------
        None.

        """
        
        self._with_rain = False
        if (len(self._rain)>0):
            self._with_rain = True
        self._same_length_ET = _check_same_length(self._ETa, self._ETc)
        self._ETa_negative = _check_negative(self._ETa)
        self._ETc_negative = _check_negative(self._ETc)
        self._rain_negative = False
        self._same_length_rain = False
        if (self._with_rain):
            self._rain_negative = _check_negative(self._rain)
            self._same_length_rain = _check_same_length(self._ETa, self._rain)
        
    def check_parameter(self):
        if (self._with_rain): 
            print ("Wrsi calculated using {} method, considering water excess.".format(self._method))
        else: 
            print ("Wrsi calculated using {} method. Water excess was not considered.".format(self._method))
        
def _check_negative(dat):
    """
    Check if there is negative value within the evapotranspiration and rainfall data

    Parameters
    ----------
    dat : list
        

    Returns
    -------
    bool: True if there is <0 value, false otherwise

    """
    for i in range(len(dat)):
        if (dat[i] < 0):
            return True
    return False
    
def _check_same_length(dat1, dat2):
    """
    Check if the two data has the same length
    Parameters
    ----------
    dat1 : list
    dat2 : list
        
    Returns
    -------
    bool : True if the two list has the same length, false otherwise.

    """
    if (len (dat1) == len(dat2)): 
        return True
    else: 
        return False



# -*- coding: utf-8 -*-
"""
Created on Sun Dec 25 13:16:29 2022

@author: farafehizoro
"""
from .wrsi import Wrsi

class wrsi_daily(Wrsi):
    """
    Class to calculate daily wrsi
    valid method: "Original" or "Modified"
    
    """
    def __init__(self, ETa, ETc, method = "Original", rain = []):
        
        Wrsi.__init__(self, ETa, ETc, method, rain)
        self.wrsi = []
        
    def calculate_wrsi_daily(self):
        """
        calculate wrsi for daily data

        Returns
        -------
        a list containing the daily wrsi data .

        """
        self.wrsi = []
        self._update_status()
        if (not self._same_length_ET):
            print ("ETa and ETc do not have the same length. \n Wrsi cannot be calculated")
            return self.wrsi
        #check if there are negative number
        
        if (self._ETa_negative or self._ETc_negative): 
            print("Negative number in ETa or ETc, please check your data. \nWrsi cannot be calculated")
            return self.wrsi
        
        #check rain data lenght and value (>0)
        if (self._with_rain):  #verify the rain data
            if(not self._same_length_rain):
                print ("Rain does not have the same length as ETa and ETc. Rain data is not considered for wrsi calculation.")
                self._rain = []
                self._with_rain = False
            if (self._rain_negative):
                print("Negative value for Rain data, please check your data. Rain data is not considered for wrsi calculation.")
                self._rain = []
                self._with_rain = False
        
        if (self._method == "Original"): #mbol tsy implémenter ko ny fahafatesan'ilay voly : to do
            self.wrsi = self._wrsi_original_daily()
        elif(self._method == "Modified"):  
            self.wrsi = self._wrsi_modified_daily()
        else: 
            print("Invalid method. \nPlease update the method to use with .update_method()")
            return self.wrsi
        
        self.check_parameter()
        return self.wrsi
        
        
        
    def _wrsi_original_daily(self):
        """
        calculate wrsi using original method for daily data

        Returns
        -------
        a list containing daily wrsi data .

        """
        wrsi_original = []
           
        wrsi_temp = 100 
        ETc_tot = sum (self._ETc)
        for i in range(len(self._ETa)): 
            if ( self._ETa[i] < self._ETc[i]): #if there was water deficit
                diff = self._ETc[i] - self._ETa[i]
                wrsi_temp = wrsi_temp - 100 * (diff / ETc_tot) #water déficit
                if(self._with_rain): 
                    if(i < 9): 
                        if(self._water_excess_daily(self._ETa[0:(i+1)], self._rain[0:(i+1)])):
                            wrsi_temp = wrsi_temp - 0.3 #3 divided by 10
                    else:
                        if(self._water_excess_daily(self._ETa[(i-9):(i+1)], self._rain[(i-9):(i+1)])):
                            wrsi_temp = wrsi_temp - 0.3 #3 divided by 10
                            
            wrsi_original.append(wrsi_temp)
        return wrsi_original
    
    def _wrsi_modified_daily(self):
        """
        calculate daily wrsi with modified method

        Returns
        -------
        wrsi: list of daily wrsi.

        """
        wrsi_modified = []
        ETa_cumul = 0
        ETc_cumul = 0
        excess_number = 0
        for i in range(len(self._ETa)):
            ETa_cumul += self._ETa[i]
            ETc_cumul += self._ETc[i]
            if(self._with_rain):
                if(i < 9): 
                    if(self._water_excess_daily(self._ETa[0:(i+1)], self._rain[0:(i+1)])):
                        excess_number += 0.1 #plus 1 jour, soit 0.1 dekad
                else:
                    if(self._water_excess_daily(self._ETa[(i-9):(i+1)], self._rain[(i-9):(i+1)])):
                        excess_number += 0.1 #plus 1 jour, soit 0.1 dekad
            wrsi_temp = 100 * ETa_cumul / ETc_cumul - (excess_number * 3)
            wrsi_modified.append(wrsi_temp)
        return wrsi_modified
        
    def _water_excess_daily(self, list_ETa, list_RR):
        """
        Method to determine if there was a water excess 
        Parameters
        ----------
        ETa : list
            evapotranpiration.
        RR : list
            evapptranspiration.

        Returns
        -------
        bool
            true if there is water excess.

        """
        sum_RR = sum(list_RR)
        sum_ETa = sum(list_ETa)
        water_excess = sum_RR - sum_ETa
        if (water_excess > 100):
            return True
        return False
    
    
        



from setuptools import setup

setup(name = 'wrsi',
      version = '0.1',
      description = 'Water requirement satisfaction index (wrsi) for crop calculator',
      packages = ['wrsi'],
      author = 'Farafehizoro Ramanjatonirina',
      zip_safe = False)

